/*
 * adl7200.h -- definitions for the adl7200 frame grabber
 *
 * Copyright (C) 1998   rubini@linux.it (Alessandro Rubini)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _LINUX_ADL7200_H_
#define _LINUX_ADL7200_H_

#include <linux/timer.h>
#include <linux/types.h>
/*
 * Macros to help debugging
 */
#define ADL_MSG "adl7200: "

#define ADL_VERSION_STRING "0.04 (Feb 1998)"

#undef PDEBUG             /* undef it, just in case */
#undef DEBUG_CODE
#ifdef ADL_DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#    define PDEBUG(fmt, args...) printk( KERN_DEBUG ADL_MSG fmt, ## args)
#    define DEBUG_CODE(code) code
#  else
     /* This one for user space */
#    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif
#else
#  define PDEBUG(fmt, args...) /* not debugging: nothing */
#  define DEBUG_CODE(code)
#endif

#undef PDEBUGG
#define PDEBUGG(fmt, args...) /* nothing: it's a placeholder */


/*
 * Then, normal stuff
 */

#define ADL_MAX_DEV 4 /* change this if needed */

#define ADL_VENDOR_ID 0x10e8
#define ADL_DEVICE_ID 0x80d8

#define ADL_CLOCK_HZ 4000000 /* 4MHz */


/* ADL registers */
#define ADL_REG_CNT0        0x00
#define ADL_REG_CNT1        0x04
#define ADL_REG_CNT2        0x08
#define ADL_REG_CNT_CTRL    0x0c
#define ADL_REG_DIN         0x10
#define ADL_REG_DOUT        0x14

#define ADL_REG_DIO_CTRL    0x18
#define    ADL_BIT_IACK         0x00000001 /* enable I_ACK */
#define    ADL_BIT_IREQ         0x00000002 /* enable I_REQ */
#define    ADL_BIT_ICNT         0x00000004 /* enable counter */
#define    ADL_BIT_IFIF         0x00000008 /* enable input FIFO */
#define    ADL_BIT_RISE         0x00000010 /* rising edge for I_TRG */
#define    ADL_BIT_TRIG         0x00000020 /* wait for trig */
#define    ADL_BIT_IENA         0x00000040 /* enable input data */

#define    ADL_BIT_OACK         0x00000080 /* enable O_ACK */
#define    ADL_BIT_OREQ         0x00000100 /* enable O_REQ */
#define    ADL_BIT_OCNT         0x00000200 /* enable counter */
#define    ADL_BIT_OFIF         0x00000400 /* enable output FIFO */
#define    ADL_BIT_OTRG         0x00000800 /* output trig (?)  */

#define    ADL_MASK_DISABLE_ALL 0x00000fff /* AND this to disable everything */
#define    ADL_MASK_DISABLE     0x0001FFFF /* still more stuff */

#define    ADL_BIT_OVER         0x00004000 /* input overrun */
#define    ADL_BIT_UNDE         0x00010000 /* output underrun */

#define ADL_REG_IRQ_CTRL    0x1c
#define    ADL_BIT_DOOACKINT    0x00000001 /* do interrupt on O_ACK */
#define    ADL_BIT_DOIREQINT    0x00000002 /* do interrupt on I_REQ */
#define    ADL_BIT_DOCNT0INT    0x00000004 /* do interrupt on counter 0 */
#define    ADL_BIT_DOCNT1INT    0x00000008 /* do interrupt on counter 1 */
#define    ADL_BIT_DOCNT2INT    0x00000010 /* do interrupt on counter 2 */

#define    ADL_BIT_ISOACKINT    0x00000020 /* interrupt is O_ACK */
#define    ADL_BIT_ISIREQINT    0x00000040 /* interrupt is I_REQ */
#define    ADL_BIT_ISCNT0INT    0x00000080 /* interrupt is counter 0 */
#define    ADL_BIT_ISCNT1INT    0x00000100 /* interrupt is counter 1 */
#define    ADL_BIT_ISCNT2INT    0x00000200 /* interrupt is counter 2 */

#define    ADL_BIT_CASCADE01    0x00000400 /* cascade counter 0 and 1 */
#define    ADL_BIT_CASCADE02    0x00000800 /* cascade counter 0 and 2 */

#define ADL_MAX_REG         0x20

/* AMCC operation registers (0x100 is a flag of mine) */
#define AMCC_MIN_REG          0x100

#define AMCC_REG_OMB1         0x100
#define AMCC_REG_OMB2         0x104
#define AMCC_REG_OMB3         0x108
#define AMCC_REG_OMB4         0x10c
#define AMCC_REG_IMB1         0x111
#define AMCC_REG_IMB2         0x114
#define AMCC_REG_IMB3         0x118
#define AMCC_REG_IMB4         0x11c
#define AMCC_REG_FIFO         0x120
#define AMCC_REG_MWAR         0x124 /* write address (reading from AMCC) */
#define AMCC_REG_MWTC         0x128 /* tranfer count for it (bytes) */

#define AMCC_REG_MRAR         0x12c /* read address (writing to AMCC) */
#define AMCC_REG_MRTC         0x130 /* tranfer count for it (bytes) */

#define AMCC_REG_MBEF         0x134

/* read and write, here, are in the way application programs think */
#define AMCC_REG_INTCSR       0x138
#define    AMCC_BIT_INTCSR_DORDINT 0x00004000 /* enable irq on wr done */
#define    AMCC_BIT_INTCSR_DOWRINT 0x00008000 /* enable irq on rd done */
#define    AMCC_BIT_INTCSR_ISRDINT 0x00040000
#define    AMCC_BIT_INTCSR_ISWRINT 0x00080000
#define    AMCC_BIT_INTCSR_ISMAINT 0x00100000 /* master abort interrupt */
#define    AMCC_BIT_INTCSR_ISTAINT 0x00200000 /* target abort interrupt */
#define    AMCC_BIT_INTCSR_ISMYINT 0x00800000 /* any, but mine for sure */

#define AMCC_REG_MCSR         0x13c
#define    AMCC_BIT_MCSR_RSTWRFIFO 0x02000000 /* p2a fifo */
#define    AMCC_BIT_MCSR_RSTRDFIFO 0x04000000 /* a2p fifo */
#define    AMCC_BIT_MSCR_DOWRREQ   0x00004000 /* req dma for p2a */
#define    AMCC_BIT_MSCR_DORDREQ   0x00000400 /* req dma for a2p */
#define    AMCC_BIT_MSCR_DOWREMPTY 0x00002000 /* only if empty (p2a) */
#define    AMCC_BIT_MSCR_DORDFULL  0x00000200 /* only if full (a2p) */
#define    AMCC_BIT_MSCR_WRPRIO    0x00001000 /* priority for p2a */
#define    AMCC_BIT_MSCR_RDPRIO    0x00000100 /* priority for a2p */

#define AMCC_MAX_REG          0x140


/* The device structure */

typedef struct Adl_Dev {
    unsigned long dmabuffer; /* physical address (independent of allocator) */
    unsigned long dmaremap;  /* virtual address, from ioremap() */
    unsigned long dmasize;   /* allocated size */
    unsigned long flags;     /* various flags */

#define ADL_FLAG_DMALOCK  0x00000001 /* active DMA locks out rd/wr */
#define ADL_FLAG_PERSIST  0x00000002 /* configuration persists on close */
#define ADL_FLAG_SINGLEO  0x00000004 /* single-openness */
#define ADL_FLAG_AUTODMA  0x00000008 /* setup DMA according to R/W buffer */
#define ADL_FLAG_READING  0x00000010 /* currently reading */
#define ADL_FLAG_WRITING  0x00000020 /* currently writing */

#define ADL_DEFAULT_FLAGS 0 /* by default everything is off */

    unsigned long irq_flags;
    unsigned long master_flags;

    struct wait_queue *inq;    /* queue for reading */
    struct wait_queue *outq;   /* queue for writing */
    struct wait_queue *closeq; /* queue for closing */
    unsigned long softbuffer;  /* buffer for kernel timers */

    unsigned long wpending;    /* pending writes (bytes) while DMA active */
    unsigned long rpending;    /* pending reads  (bytes) while DMA active */

    unsigned long rdhead; /* for softtimers */
    unsigned long rdtail;
    unsigned long wrhead;
    unsigned long wrtail;

    __u32 io;        /* the I/O range */
    __u32 io_amcc;   /* the I/O range for OP registers of the S5933 */
    __u32 irq_count; /* number of interrupts received */
    __u32 counter;   /* counter for the 8254 clock */
#define ADL_DEFAULT_COUNTER (ADL_CLOCK_HZ/8000) /* 8kHz */

    __u32 dma_count;         /* some statistics... */
    __u32 fifo_overr_count;
    __u32 fifo_under_count;
    __u32 unused1;

    int usage;               /* number of opens */
    int unused2;
    int dma_requested_size;  /* size of the buffer */
    int hertz;               /* frequency, for soft timing */
#define ADL_DEFAULT_DMA_SIZE (32<<10) /* 32k */
#define ADL_DEFAULT_HZ 10

    __u8 bus;                /* pci bus for this board */
    __u8 device;             /* pci device and function */
    __u8 irq;                /* irq number from pci config */
    __u8 irq_active;         /* currently used. 0 if unused */

    struct timer_list timer; /* for soft timekeeping and xfer-while-DMA */

} Adl_Dev;

/* This structure is needed to help I/O */

struct Adl_Io_Struct {
    unsigned long val;
    unsigned long io; 
};

#include <sys/ioctl.h>

#define ADL_IOC_MAGIC  'd' /* Use 'd' as magic number */

#define ADL_IOCRESET        _IO(ADL_IOC_MAGIC, 0)
#define ADL_IOCHARDRESET    _IO(ADL_IOC_MAGIC, 1) /* debugging tool */

/* _IOW 'cause program writes I/O address */
#define ADL_IOCIN           _IOW(ADL_IOC_MAGIC, 2, unsigned long)
#define ADL_IOCOUT          _IOW(ADL_IOC_MAGIC, 3, struct Adl_Io_Struct)

/* flags: no need to access individual flags: apps can get, change, set */
#define ADL_IOCGFLAGS       _IOR(ADL_IOC_MAGIC, 4, unsigned long)
#define ADL_IOCSFLAGS       _IOW(ADL_IOC_MAGIC, 5, unsigned long)

/* timer counter */
#define ADL_IOCGTIMER       _IOR(ADL_IOC_MAGIC, 6, unsigned long)
#define ADL_IOCSTIMER       _IOW(ADL_IOC_MAGIC, 7, unsigned long)

/* hertz (kernel timers) */
#define ADL_IOCGHERTZ       _IOR(ADL_IOC_MAGIC, 8, unsigned long)
#define ADL_IOCSHERTZ       _IOW(ADL_IOC_MAGIC, 9, unsigned long)

/* dma size */
#define ADL_IOCGDMASIZE     _IOR(ADL_IOC_MAGIC,10, unsigned long)
#define ADL_IOCSDMASIZE     _IOW(ADL_IOC_MAGIC,11, unsigned long)

/* statistics */
#define ADL_IOCGDMACOUNT    _IOR(ADL_IOC_MAGIC,12, unsigned long)
#define ADL_IOCGUNDERRUN    _IOR(ADL_IOC_MAGIC,13, unsigned long)
#define ADL_IOCGOVERRUN     _IOR(ADL_IOC_MAGIC,14, unsigned long)

#define ADL_IOC_MAXNR                          14


#ifdef __KERNEL__ /*========================================================*/

/* This isn't meant to be seen by userspace */

#include <linux/ioctl.h>
#include <linux/types.h>
#include <linux/delay.h>

/* version dependencies have been confined to a separate file */

#include "sysdep-2.1.h"

/* functions that are extern to some of the sources */
int adl_shutdown(Adl_Dev *dev);
int adl_bringup(Adl_Dev *dev);
int adl_allocate_dma(Adl_Dev *dev, int kilobytes, int prio);
void adl_free_dma(Adl_Dev *dev);
int adl_run_dma(Adl_Dev *dev, u32 bitstoset, u32 bitstoclear);
int adl_set_clock(Adl_Dev *dev);

#ifdef __USE_OLD_SELECT__
int adl_poll (struct inode *inode, struct file *filp,
                  int mode, select_table *table);
#else
unsigned int adl_poll (struct file *filp, poll_table *table);
#endif
int adl_ioctl (struct inode *inode, struct file *filp,
                 unsigned int cmd, unsigned long arg);





/* Use macros to access registers: only 32-bit access is allowed */

/* general macros */
#define ADL_IN32(d,r) inl((r)&0x100 ? (d)->io_amcc+((r)&0xFF) : (d)->io+(r))
#define ADL_OUT32(d,v,r) outl(v,(r)&0x100 ? \
			      (d)->io_amcc+((r)&0xFF) : (d)->io+(r))

/* first I/O range: L for "local" */
#define ADL_IN32_L(d,r) (inl((d)->io+(r)))
#define ADL_OUT32_L(d,v,r) (outl(v,(d)->io+(r)))

/* second I/O range: A for "AMCC" -- this also works with bytes, bah! */
#define ADL_IN32_A(d,r) (inl((d)->io_amcc+((r)&0xFF)))
#define ADL_OUT32_A(d,v,r) (outl(v,(d)->io_amcc+((r)&0xFF)))
#define ADL_IN8_A(d,r) (inb((d)->io_amcc+((r)&0xFF)))
#define ADL_OUT8_A(d,v,r) (outb(v,(d)->io_amcc+((r)&0xFF)))

/* configuration of the soft driver */

#define ADL_SFT_ORDER 0 /* order of allocation 0=1page, 2=4pages etc */

#define ADL_SFT_MASK ((PAGE_SIZE << ADL_SFT_ORDER)-1)

#define ADL_SFT_REMPTY(dev) ((dev)->rdhead == (dev)->rdtail)
#define ADL_SFT_RFULL(dev) ((((dev)->rdtail-(dev)->rdhead)&ADL_SFT_MASK)==1)

#define ADL_SFT_WEMPTY(dev) ((dev)->wrhead==(dev)->wrtail)
#define ADL_SFT_WFREE(dev) (((dev)->wrtail-(dev)->wrhead-1)&ADL_SFT_MASK)
#define ADL_SFT_WFULL(dev) (!ADL_SFT_WFREE(dev))

#define ADL_SFT_THRESHOLD  400 /* at least 400 free before one writes */

/*
 * Split minors in two parts
 */
#define TYPE(dev)   (MINOR(dev) >> 4)  /* high nibble */
#define NUM(dev)    (MINOR(dev) & 0xf) /* low  nibble */

enum px_types {
    ADL_RAW = 0,   /* read/write single items */
    ADL_DMA,       /* read/write using DMA and internal timer */
    ADL_TRG,       /* use DMA and REQ/ACK I/O */
    ADL_SFT,       /* read/write using sw timekeeping (kernel timers) */
    ADL_ASC        /* ascii device: write commands and read status (bleah) */
};

#define ADL_MAX_TYPE 4 /* 0-4 is valid */

extern struct file_operations adl_raw_fops;
extern struct file_operations adl_dma_fops;
extern struct file_operations adl_trg_fops;
extern struct file_operations adl_sft_fops;
extern struct file_operations adl_asc_fops;

extern struct file_operations *adl_fops[];   /* one fops for type */

/*
 * The different configurable parameters
 */

#define    ADL_MAJOR 60   /* not dynamic major by default -- FIXME -- */
extern int adl_major;

extern Adl_Dev adl_devices[ADL_MAX_DEV];

#define ADL_KEEPALIVE 10 /* number of second of activity after close */

#ifdef ADL_DEBUG 
#  define ADL_USE_PROC
#endif

#ifndef min
#  define min(a,b) ((a)<(b) ? (a) : (b))
#endif

#else /* !__KERNEL__ =======================================================*/

/*
 * These macros are wrappers to ioctl()
 * They use gcc extensions to the C language, but every Linux runs gcc
 */

/* Reset. Hardreset not wrapped, as it is only a debugging tool */

#define Adl_Reset(fd) (ioctl((fd), ADL_IOCRESET))

/* command-based information */

#define Adl_In(fd,io)    ioctl((fd),ADL_IOCIN,(unsigned long)io)

extern inline int Adl_Out(int fd, unsigned long val,
                             unsigned long io)
{
    struct Adl_Io_Struct str;

    str.val = val;
    str.io = io;
    return ioctl(fd,ADL_IOCOUT,&str);
}

#endif /* __KERNEL__ */

#endif /* _LINUX_ADL7200_H_ */
